bfgs.lm.grp <- function(x0, XtX, Xty, lambda, pk, eps) {
  res <- optim(x0, method="L-BFGS-B",
               fn     = L.group.gaussian,
               gr     = dL.group.gaussian,
               XtX    = XtX,
               Xty    = Xty,
               lambda = lambda,
               pk     = pk,
               control= list(pgtol=eps,factr=eps))
  return(list(i=res$counts[1],xk=res$par))
}

bfgs.lm.coo <- function(x0, XtX, Xty, lambda, pk, eps, lower, upper) {
  res <- optim(x0, method="L-BFGS-B",
               fn     = L.coop.gaussian,
               gr     = dL.coop.gaussian,
               XtX    = XtX,
               Xty    = Xty,
               lambda = lambda,
               pk     = pk,
               lower = lower,
               upper = upper,
               control= list(factr=eps,pgtol=eps))
  return(list(i=res$counts[1],xk=res$par))
}

ista.lm.grp <- function(x0, XtX, Xty, lambda, pk, eps) {
  p <- length(x0)
  K <- length(pk)
  return(.C("ISTA_LM_GRP",
            as.integer(p),
            as.integer(K),
            as.integer(pk),
            as.double(XtX),
            as.double(Xty),
            as.double(lambda),
            as.double(max(eigen(XtX)$values,TRUE,TRUE)),
            as.integer(10000), # max # of iteration
            as.double(eps),
            x0 = as.double(x0),
            xk = as.double(x0),
            i  = as.integer(0)))
}

ista.lm.coo <- function(x0, XtX, Xty, lambda, pk, eps, lower, upper) {
  p <- length(x0)
  K <- length(pk)
  return(.C("ISTA_LM_COO",
            as.integer(p),
            as.integer(K),
            as.integer(pk),
            as.double(XtX),
            as.double(Xty),
            as.double(lambda),
            as.double(max(eigen(XtX)$values,TRUE,TRUE)),
            as.integer(10000), # max # of iteration
            as.double(eps),
            x0 = as.double(x0),
            xk = as.double(x0),
            i  = as.integer(0)))
}

fista.lm.grp <- function(x0, XtX, Xty, lambda, pk, eps) {
  p <- length(x0)
  K <- length(pk)
  return(.C("FISTA_LM_GRP",
            as.integer(p),
            as.integer(K),
            as.integer(pk),
            as.double(XtX),
            as.double(Xty),
            as.double(lambda),
            as.double(max(eigen(XtX)$values,TRUE,TRUE)),
            as.integer(10000), # max # of iteration
            as.double(eps),
            x0 = as.double(x0),
            xk = as.double(x0),
            i  = as.integer(0)))
}

fista.lm.coo <- function(x0, XtX, Xty, lambda, pk, eps, lower, upper) {
  p <- length(x0)
  K <- length(pk)
  return(.C("FISTA_LM_COO",
            as.integer(p),
            as.integer(K),
            as.integer(pk),
            as.double(XtX),
            as.double(Xty),
            as.double(lambda),
            as.double(max(eigen(XtX)$values,TRUE,TRUE)),
            as.integer(10000), # max # of iteration
            as.double(eps),
            x0 = as.double(x0),
            xk = as.double(x0),
            i  = as.integer(0)))
}


bfgs.lrm.grp <- function(x0, X, y, lambda, pk, eps) {
  res <- optim(x0, method="L-BFGS-B",
               fn     = L.group.binomial,
               gr     = dL.group.binomial,
               X      = X,
               y      = y,
               lambda = lambda,
               pk     = pk,
               control= list(factr=eps,pgtol=eps))
  res$par[abs(res$par) < eps] <- 0
  return(list(i=res$counts[1],xk=res$par))
}

bfgs.lrm.coo <- function(x0, X, y, lambda, pk, eps, lower, upper) {
  if (length(x0) != length(lower)) {
    lower <- c(-Inf,lower)
    upper <- c( Inf,upper)
  }  
  res <- optim(x0, method="L-BFGS-B",
               fn     = L.coop.binomial,
               gr     = dL.coop.binomial,
               X      = X,
               y      = y,
               lambda = lambda,
               pk     = pk,
               lower = lower,
               upper = upper,
               control= list(factr=eps,pgtol=eps))
  res$par[abs(res$par) < eps] <- 0
  return(list(i=res$counts[1],xk=res$par))
}
